

set -ex



test -f $PREFIX/include/git2.h
test -f $PREFIX/include/git2/annotated_commit.h
test -f $PREFIX/include/git2/attr.h
test -f $PREFIX/include/git2/blame.h
test -f $PREFIX/include/git2/blob.h
test -f $PREFIX/include/git2/branch.h
test -f $PREFIX/include/git2/buffer.h
test -f $PREFIX/include/git2/checkout.h
test -f $PREFIX/include/git2/apply.h
test -f $PREFIX/include/git2/cert.h
test -f $PREFIX/include/git2/cherrypick.h
test -f $PREFIX/include/git2/clone.h
test -f $PREFIX/include/git2/commit.h
test -f $PREFIX/include/git2/common.h
test -f $PREFIX/include/git2/config.h
test -f $PREFIX/include/git2/cred_helpers.h
test -f $PREFIX/include/git2/credential.h
test -f $PREFIX/include/git2/credential_helpers.h
test -f $PREFIX/include/git2/deprecated.h
test -f $PREFIX/include/git2/describe.h
test -f $PREFIX/include/git2/diff.h
test -f $PREFIX/include/git2/errors.h
test -f $PREFIX/include/git2/filter.h
test -f $PREFIX/include/git2/global.h
test -f $PREFIX/include/git2/graph.h
test -f $PREFIX/include/git2/ignore.h
test -f $PREFIX/include/git2/index.h
test -f $PREFIX/include/git2/indexer.h
test -f $PREFIX/include/git2/mailmap.h
test -f $PREFIX/include/git2/merge.h
test -f $PREFIX/include/git2/message.h
test -f $PREFIX/include/git2/net.h
test -f $PREFIX/include/git2/notes.h
test -f $PREFIX/include/git2/object.h
test -f $PREFIX/include/git2/odb.h
test -f $PREFIX/include/git2/odb_backend.h
test -f $PREFIX/include/git2/oid.h
test -f $PREFIX/include/git2/oidarray.h
test -f $PREFIX/include/git2/pack.h
test -f $PREFIX/include/git2/patch.h
test -f $PREFIX/include/git2/pathspec.h
test -f $PREFIX/include/git2/proxy.h
test -f $PREFIX/include/git2/rebase.h
test -f $PREFIX/include/git2/refdb.h
test -f $PREFIX/include/git2/reflog.h
test -f $PREFIX/include/git2/refs.h
test -f $PREFIX/include/git2/refspec.h
test -f $PREFIX/include/git2/remote.h
test -f $PREFIX/include/git2/repository.h
test -f $PREFIX/include/git2/reset.h
test -f $PREFIX/include/git2/revert.h
test -f $PREFIX/include/git2/revparse.h
test -f $PREFIX/include/git2/revwalk.h
test -f $PREFIX/include/git2/signature.h
test -f $PREFIX/include/git2/stash.h
test -f $PREFIX/include/git2/status.h
test -f $PREFIX/include/git2/stdint.h
test -f $PREFIX/include/git2/strarray.h
test -f $PREFIX/include/git2/submodule.h
test -f $PREFIX/include/git2/sys/alloc.h
test -f $PREFIX/include/git2/sys/commit.h
test -f $PREFIX/include/git2/sys/config.h
test -f $PREFIX/include/git2/sys/cred.h
test -f $PREFIX/include/git2/sys/credential.h
test -f $PREFIX/include/git2/sys/diff.h
test -f $PREFIX/include/git2/sys/filter.h
test -f $PREFIX/include/git2/sys/hashsig.h
test -f $PREFIX/include/git2/sys/index.h
test -f $PREFIX/include/git2/sys/mempack.h
test -f $PREFIX/include/git2/sys/merge.h
test -f $PREFIX/include/git2/sys/odb_backend.h
test -f $PREFIX/include/git2/sys/openssl.h
test -f $PREFIX/include/git2/sys/path.h
test -f $PREFIX/include/git2/sys/refdb_backend.h
test -f $PREFIX/include/git2/sys/reflog.h
test -f $PREFIX/include/git2/sys/refs.h
test -f $PREFIX/include/git2/sys/repository.h
test -f $PREFIX/include/git2/sys/stream.h
test -f $PREFIX/include/git2/sys/transport.h
test -f $PREFIX/include/git2/tag.h
test -f $PREFIX/include/git2/trace.h
test -f $PREFIX/include/git2/transaction.h
test -f $PREFIX/include/git2/transport.h
test -f $PREFIX/include/git2/tree.h
test -f $PREFIX/include/git2/types.h
test -f $PREFIX/include/git2/version.h
test -f $PREFIX/include/git2/worktree.h
test -f $PREFIX/lib/libgit2.so
exit 0
